/*

	DFApplication.cpp
	
	Copyright 1996 Thorsten Seitz, All Rights Reserved
	
*/

#include "DFApplication.h"


// class DFActivationWindow

#pragma mark ## ACTIVATION_WINDOW

DFActivationWindow::DFActivationWindow(void)
	: BWindow(BRect(-10,-10,-9,-9), "", B_TITLED_WINDOW, 0, B_ALL_WORKSPACES)
{
	fAppActive = TRUE;
}


void
DFActivationWindow::WindowActivated(bool flag)
{
	if (flag) {
		if (!fAppActive) {
			be_app->Activate();
		}
		else {
//			Activate(FALSE);
		}
	}	
}


void
DFActivationWindow::SetAppActive(bool flag)
{
	fAppActive = flag;
}


// class DFApplication

#pragma mark ## APPLICATION

DFApplication::DFApplication(ulong signature)
	: BApplication(signature), hidden(FALSE), windowList(5)
{
	long tryMW;
	
	fActivationWindow = new DFActivationWindow();
	
	// try to start the miniWindowDaemon
	
	fMWD = FALSE;
	fMW = FALSE;
	long result = be_roster->Launch('?MWD');
	
	switch (result) {
	
	case B_BAD_VALUE:
	case B_ERROR:
	case B_LAUNCH_FAILED:
		tryMW = (new BAlert("MiniWindowDaemon launch failure",
		"The MiniWindowDaemon could not be launched on your system.\n"
		"If you want to use mini windows instead of title tabs "
		"for minimized windows, it is needed to administrate "
		"their arrangement on your desktop.\nMini windows can display "
		"status informations for their respective windows."
		"If you would like to use this feature, please get "
		"MiniWindowDaemon from BeWare.\n"
		"You may try to use mini windows nevertheless:\n"
		"In this case each mini window is put on the upper "
		"left of your desktop just to the right of the dock "
		"the first time it is created.", 
		"Try mini windows",
		"Use normal minimization", NULL,
		B_WIDTH_FROM_LABEL, B_WARNING_ALERT))->Go();
		if (tryMW == 0) fMW = TRUE;
		break;
		
	case B_NO_ERROR:
	case B_ALREADY_RUNNING:
	{
		fMWD = TRUE;
		fMW = TRUE;
		BMessenger mwd('?MWD');
		mwd.SendMessage(DF_ADD);
		break;
	}
		
	default:
		break;
	}
}


DFApplication::~DFApplication()
{
	if (fMWD) {
		BMessenger mwd('?MWD');
		if (mwd.Error() == B_NO_ERROR)
			mwd.SendMessage(DF_QUIT);
	}
}


void
DFApplication::AddStandardMenuItems(void)
{
	long 		index;
	BMenuItem  	*item;
	BPopUpMenu 	*menu = MainMenu();
	
	index = menu->CountItems() - 1;
	
	menu->AddItem(new BSeparatorItem(), index++);	
	
	item = new BMenuItem("Lower", new BMessage(DF_LOWER));
	item->SetTarget(be_app);
	menu->AddItem(item, index++);

	item = new BMenuItem("Raise", new BMessage(DF_RAISE));
	item->SetTarget(be_app);
	menu->AddItem(item, index++);
	
	item = new BMenuItem("Hide", new BMessage(DF_HIDE));
	item->SetTarget(be_app);
	menu->AddItem(item, index++);
	
	item = new BMenuItem("Hide all others", 
				new BMessage(DF_HIDE_ALL_OTHERS));
	item->SetTarget(be_app);
	menu->AddItem(item, index++);	
	
	menu->AddItem(new BSeparatorItem(), index++);	
}	


void
DFApplication::Activate(void)
{
	DFWindow				*window;
	long     				i=0;
	bool					success = FALSE;
	
	if (windowList.CountItems() == 0) return;
	
	{
		StLocker<DFLockedList> 	lock(&windowList);

		// try to find a window in the current workspace which is
		// neither hidden nor minimized
	
		while( window = static_cast<DFWindow*>(windowList.ItemAt(i++)) )
			if (!window->IsMinimized() 
				&& !window->IsHidden()
				&& (window->Workspaces() & 1<<current_workspace())) {

				success = TRUE;
				break;
			}
#if(0)	
		// if unsuccesful try to find a window in the current workspace
		// which may be hidden or minimized
	
		i = 0;
		if (!success) {
			while( window = static_cast<DFWindow*>(windowList.ItemAt(i++)) )
				if (window->Workspaces() & 1<<current_workspace()) {
					
					success = TRUE;
					break;
				}
		}
#endif	
		// if unsuccesful activate window #0 and put it in the current
		// workspace
		
		if (!success) {
			window = static_cast<DFWindow*>(windowList.ItemAt(0));
			window->SetWorkspaces(window->Workspaces() 
									| 1<<current_workspace());
		}
		
	}

	if (window->IsMinimized())
		window->Minimize(FALSE);
	if (window->IsHidden())
		window->Show();
}		



void
DFApplication::AppActivated(bool flag)
{
	if (flag && hidden) Hide(FALSE);
	if (flag) 
		Activate();
	else
		fActivationWindow->Activate(TRUE);

	fActivationWindow->SetAppActive(flag);
}


void
DFApplication::MessageReceived(BMessage *message)
{
	switch (message->what) {
	
	case DF_LOWER:
		Raise(FALSE);
		break;
		
	case DF_RAISE:
		Raise(TRUE);
		break;
		
	case DF_HIDE:
		Hide(TRUE);
		break;
		
	case DF_UNHIDE:
		Hide(FALSE);
		break;
		
	case DF_HIDE_ALL_OTHERS:
		HideAllOthers();
		break;
		
	default:
		inherited::MessageReceived(message);
		break;
	}
}


void
DFApplication::ReadyToRun(void)
{
	inherited::ReadyToRun();
	
	AddStandardMenuItems();
}


void
DFApplication::Raise(bool raise)
{
	DFWindow 				*window;
	long     				i=0;
	StLocker<DFLockedList> 	lock(&windowList);

	while( window = static_cast<DFWindow*>(windowList.ItemAt(i++)) )
		window->Activate(raise);
}
	

void
DFApplication::Hide(bool hide)
{
	DFWindow   				*window;
	long       				i=0;
	StLocker<DFLockedList> 	lock(&windowList);

	if (hidden == hide) return;
	
	hidden = hide;
	while( window = static_cast<DFWindow*>(windowList.ItemAt(i++)) )
		if (hide)
			window->Hide();
		else
			window->Show();
}


void
DFApplication::HideAllOthers(void)
{
	team_id  who, me = Team();
	long     i;
	BList    teams;
	app_info appInfo;
	
	
	be_roster->GetAppList(&teams);
	
	for (i=0; i<teams.CountItems(); i++) {
		who = static_cast<team_id>(teams.ItemAt(i));
		if (who != me) {
			be_roster->GetRunningAppInfo(who, &appInfo);
			BMessenger hermes(appInfo.signature, who);
			hermes.SendMessage(DF_HIDE);
		}
	}
}


void
DFApplication::Broadcast(BMessage *msg)
{
	StLocker<DFLockedList> lock(&windowList);
	
	DFWindow	*window;
	long		i=0;
	
	while( window = static_cast<DFWindow*>(windowList.ItemAt(i++)) )
		window->PostMessage(new BMessage(msg));
		
	delete msg;
}
	

void
DFApplication::RegisterWindow(DFWindow *window)
{
	StLocker<DFLockedList> lock(&windowList);

	windowList.AddItem(static_cast<void*>(window));
}


void
DFApplication::UnregisterWindow(DFWindow *window)
{
	StLocker<DFLockedList> lock(&windowList);

	windowList.RemoveItem(static_cast<void*>(window));
	if (windowList.IsEmpty())
		be_app->PostMessage(B_QUIT_REQUESTED);
}


void
DFApplication::WindowActivated(DFWindow *window)
{
	StLocker<DFLockedList> lock(&windowList);

	windowList.RemoveItem(static_cast<void*>(window));
	windowList.AddItem(static_cast<void*>(window), 0);
}
	