/*

	DFApplication.h
	
	Copyright 1996 Thorsten Seitz, All Rights Reserved
	
*/

#ifndef DF_APPLICATION_H
#define DF_APPLICATION_H

#include "DFWindow.h"
#include "StLocker.h"

enum {
	DF_REMOVE_ITEM = '?RIT',
	DF_LOCATION = '?LOC',
	DF_MOVED = '?MVD',
	DF_QUIT = '?QUT',
	DF_ADD = '?ADD',
	DF_LOWER = '?LOW',
	DF_RAISE = '?RAI',
	DF_HIDE = '?HID',
	DF_HIDDEN = '?HDN',
	DF_UNHIDE = '?UHD',
	DF_HIDE_ALL_OTHERS = '?HAO'
};


class
DFLockedList : public BList
{

public:
					DFLockedList(long size = 20) : BList(size) {}
					DFLockedList(const BList &list) : BList(list) {}

	virtual			~DFLockedList() {}

	bool			Lock() { return fLock.Lock(); }
	void			Unlock() { fLock.Unlock(); }
	
private:
	BLocker			fLock;
};


class
DFActivationWindow : public BWindow
{

public:
					DFActivationWindow(void);
	virtual			~DFActivationWindow() {}

	virtual	void	WindowActivated(bool flag);
			void	SetAppActive(bool flag);
			
private:
			bool	fAppActive;
};


class
DFApplication : public BApplication
{

public:
						DFApplication(ulong signature);
	virtual 			~DFApplication();
	
	virtual	void		AddStandardMenuItems(void);
	
	virtual void		Activate(void);
	virtual void		AppActivated(bool flag);
	
	virtual void 		MessageReceived(BMessage *message);
	virtual void		ReadyToRun(void);
	
	virtual void		Raise(bool raise);
	virtual void 		Hide(bool hide);
	virtual void 		HideAllOthers(void);
	
	virtual void		Broadcast(BMessage *msg);
	
			void    	RegisterWindow(DFWindow *window);
			void    	UnregisterWindow(DFWindow *window);
			void		WindowActivated(DFWindow *window);

			bool		MiniWindows(void) const { return fMW; }

protected:
	bool				fMWD;
	bool				fMW;
	bool				hidden;
	DFLockedList		windowList;
	DFActivationWindow	*fActivationWindow;
};

#endif
