/*
	
	DFWindow.cpp
	
	Copyright 1996 Thorsten Seitz, All Rights Reserved
	
*/

#include "DFApplication.h"
#include "DFWindow.h"
#include "DFIconButton.h"
#include "colors.h"
#include "get_icon.h"


// class DFMiniWindow

#pragma mark ## MINI_WINDOW

DFMiniWindow::DFMiniWindow(DFWindow *bigWindow, ulong workspaces)
	: BWindow(BRect(DOCK_WIDTH,TOP,DOCK_WIDTH + 47,TOP + 45), 
				bigWindow->Title(), 
				WINDOW_TYPE,
				B_NOT_CLOSABLE | B_NOT_ZOOMABLE | B_NOT_MINIMIZABLE
				| B_NOT_RESIZABLE | B_WILL_ACCEPT_FIRST_CLICK,
				workspaces)
{
	fBig = bigWindow;
	AddChild(new DFMiniView(bigWindow));
}


void
DFMiniWindow::Show(void)
{
	BMessage *request, *reply;
	
	if (IsHidden()) {

		// get location
		
		BPoint point = Frame().LeftTop();
		
		BMessenger mwd('?MWD');
		if (mwd.Error() == B_NO_ERROR) {
			request = new BMessage(DF_LOCATION);
			request->AddObject("mini", this);
			request->AddPoint("point", point);
			mwd.SendMessage(request, &reply);
			point = reply->FindPoint("point");
		}
		MoveTo(point.x, point.y);
	}
	
	inherited::Show();
	Activate(FALSE);
}


void
DFMiniWindow::Hide(void)
{
	BMessenger mwd('?MWD');
	if (mwd.Error() == B_NO_ERROR) {
		BMessage *msg = new BMessage(DF_REMOVE_ITEM);
		msg->AddObject("mini", this);
		mwd.SendMessage(msg);
	}
	
	inherited::Hide();
}


void
DFMiniWindow::WindowActivated(bool flag)
{
	if (flag) Activate(FALSE);
}


void		
DFMiniWindow::WorkspacesChanged(ulong oldWS, ulong newWS)
{
	switch (newWS) {
	
	case B_CURRENT_WORKSPACE:
		newWS = 1<<current_workspace();
		break;
		
	case B_ALL_WORKSPACES:
		newWS = (1<<count_workspaces()) - 1;
		break;
	}

	if (newWS != fBig->Workspaces())	
		fBig->SetWorkspaces(newWS);
}


// class DFMiniView

#pragma mark ## MINI_VIEW

DFMiniView::DFMiniView(DFWindow *bigWindow)
	: BView(BRect(0,0,47,45), "", B_FOLLOW_NONE, B_WILL_DRAW)
{
	DFIconButton *close;
	fBig = bigWindow;

	SetViewColor(ltGray);
	SetLowColor(ltGray);
	
	close = new DFIconButton(bigWindow, BRect(0,0,15,15), "", 
				'CLS0', 'CLS1', '?MWD', 0, 0, 0,
				new BMessage(B_QUIT_REQUESTED));
	close->SetTarget(fBig);
	AddChild(close);
	
	fTitle = new BStringView(BRect(0, 32, 47, 45), "", fBig->Title());
	fTitle->SetViewColor(ltGray);
	fTitle->SetLowColor(ltGray);
	AddChild(fTitle);

	fIcon = new BBitmap(BRect(0.0, 0.0, 31.0, 31.0), B_COLOR_8_BIT);
	fSmallIcon = NULL;
	
	SetMiniIcon();
}


DFMiniView::~DFMiniView()
{
	delete fIcon;
	delete fSmallIcon;
}


void
DFMiniView::SetMiniIcon(const BBitmap *bmp)
{
	app_info 	info;
	
	if (bmp) {
		fIcon->SetBits(bmp->Bits(), fIcon->BitsLength(), 0,
						bmp->ColorSpace());
	}	
	else {	
		if (be_app->GetAppInfo(&info) != B_NO_ERROR)
			info.signature = '    ';
		
		get_icon('BAPP', info.signature, "largeBits", fIcon);
	}

	if (Window()) Window()->UpdateIfNeeded();
}


void
DFMiniView::SetSmallMiniIcon(const BBitmap *bmp)
{
	if (bmp) {
		delete fSmallIcon;
		fSmallIcon = new BBitmap(BRect(0,0,15,15), B_COLOR_8_BIT);
		fSmallIcon->SetBits(bmp->Bits(), fSmallIcon->BitsLength(), 0,
						bmp->ColorSpace());
	}
	else
		fSmallIcon = NULL;

	if (Window()) Window()->UpdateIfNeeded();
}


void
DFMiniView::Draw(BRect)
{
	SetDrawingMode(B_OP_COPY);
	FillRect(Bounds(), B_SOLID_LOW);
	SetDrawingMode(B_OP_OVER);
	DrawBitmap(fIcon, BPoint(16,0));
	
	if (fSmallIcon) {
		DrawBitmap(fSmallIcon, BPoint(0,16));
	}
}


void
DFMiniView::MouseDown(BPoint where)
{
	ulong 		buttons;
	screen_info info;
	
	Window()->Activate(FALSE);
	
	if (Window()->CurrentMessage()->FindLong("clicks") == 2) {
		fBig->Minimize(FALSE);
		return;
	}
	
	get_screen_info(&info);
	
	BPoint delta = ConvertToScreen(where) - Window()->Frame().LeftTop();
	
	while (1) {
		snooze(30.0 * 1000.0);
		GetMouse(&where, &buttons);
		
		if (buttons == 0) break;
		
		where = ConvertToScreen(where);
		
		if (buttons & B_SECONDARY_MOUSE_BUTTON) {
			where.x = floor((where.x - DOCK_WIDTH) / GRID_X) * GRID_X + DOCK_WIDTH;
			where.y = floor((where.y - TOP) / GRID_Y) * GRID_Y + TOP;
		}
		else
			where -= delta;
		
		if (where.x < DOCK_WIDTH) where.x = DOCK_WIDTH;
		if (where.y < TOP) where.y = TOP;
		if (where.x > info.frame.right - GRID_X + RIGHT_OFFSET) 
			where.x = info.frame.right - GRID_X + RIGHT_OFFSET;
		if (where.y > info.frame.bottom - GRID_Y + BOTTOM_OFFSET) 
			where.y = info.frame.bottom - GRID_Y + BOTTOM_OFFSET;
			
		Window()->MoveTo(where.x, where.y);
		Window()->UpdateIfNeeded();
		
	}
	
	BMessenger mwd('?MWD');
	if (mwd.Error() == B_NO_ERROR) {
		BMessage *msg = new BMessage(DF_MOVED);
		msg->AddPoint("point", Window()->Frame().LeftTop());
		msg->AddObject("mini", Window());
		mwd.SendMessage(msg);
	}
}


// class DFFilter

#pragma mark ## FILTER

DFFilter::DFFilter(void)
	: BMessageFilter(B_PROGRAMMED_DELIVERY, B_ANY_SOURCE,
					B_MOUSE_DOWN)
{
}


filter_result
DFFilter::Filter(BMessage *, BHandler **target)
{
	BView *view = static_cast<BView*>(*target);	

	if (modifiers() & B_COMMAND_KEY) {
		view->Window()->Activate(FALSE);
		return B_SKIP_MESSAGE;
	}
	else
		return B_DISPATCH_MESSAGE;
}


// class DFWindow

#pragma mark ## WINDOW

DFWindow::DFWindow(
				BRect frame, 
				const char *title,
				window_type type, 
				ulong flags, 
				ulong workspaces)
	: BWindow(frame, title, type, flags, workspaces)
{
	DFApplication 	*df_app = static_cast<DFApplication*>(be_app);
	
	df_app->RegisterWindow(this);

	if (df_app->MiniWindows())
		fMini = new DFMiniWindow(this, workspaces);
	else
		fMini = NULL;
		
	fMinimized = FALSE;
	
	fFilter = new DFFilter();
	
	StLocker<DFWindow> lock(this);
	AddCommonFilter(fFilter);
}



DFWindow::~DFWindow()
{
	DFApplication 	*df_app = static_cast<DFApplication*>(be_app);
	
	df_app->UnregisterWindow(this);
	
	if (fMini) fMini->Quit();
}


void
DFWindow::Minimize(bool minimize)
{	
	if (!fMini) {  // MiniWindows active?
		inherited::Minimize(minimize);
		return;
	}
	
	if (fMinimized == minimize) return;
	
	fMinimized = minimize;
	
	if (!minimize) {
//		SetWorkspaces(fMini->Workspaces() | 1<<current_workspace());
		fMini->Hide();
		if (IsHidden()) inherited::Show();
		return;
	}
	
//	fMini->SetWorkspaces(Workspaces());
	if (!IsHidden()) inherited::Hide();
	fMini->Show();
}


void
DFWindow::Hide(void)
{
	if (fMinimized)
		fMini->Hide();
	else
		inherited::Hide();
}


void
DFWindow::Show(void)
{
	if (fMinimized)
		fMini->Show();
	else
		inherited::Show();
}


void
DFWindow::WindowActivated(bool flag)
{
	DFApplication 	*df_app = static_cast<DFApplication*>(be_app);
	
	if (flag) df_app->WindowActivated(this);
	
	inherited::WindowActivated(flag);
}
	

void		
DFWindow::WorkspacesChanged(ulong oldWS, ulong newWS)
{
	switch (newWS) {
	
	case B_CURRENT_WORKSPACE:
		newWS = 1<<current_workspace();
		break;
		
	case B_ALL_WORKSPACES:
		newWS = (1<<count_workspaces()) - 1;
		break;
	}

	if (newWS != fMini->Workspaces())	
		fMini->SetWorkspaces(newWS);
}


