/*
	
	DFWindow.h
	
	Copyright 1996 Thorsten Seitz, All Rights Reserved.
	
*/

#ifndef DF_WINDOW_H
#define DF_WINDOW_H


// B_TITLED_WINDOW
#if (0)
#define WINDOW_TYPE B_TITLED_WINDOW
#define DOCK_WIDTH 80.0
#define TOP 5.0
#define GRID_X 60.0
#define GRID_Y 78.0
#define RIGHT_OFFSET 7.0
#define BOTTOM_OFFSET 28.0
#else
// B_BORDERED_WINDOW
#define WINDOW_TYPE B_BORDERED_WINDOW
#define DOCK_WIDTH 76.0
#define TOP 1.0
#define GRID_X 	52.0
#define GRID_Y	50.0
#define RIGHT_OFFSET 6.0
#define BOTTOM_OFFSET 6.0
#endif
 

class DFWindow;


// class DFMiniView

#pragma mark ## MINI_VIEW

class
DFMiniView : public BView
{

public:
						DFMiniView(
							DFWindow *bigWindow);
							
	virtual				~DFMiniView();
	
	virtual void		Draw(BRect);
	virtual void		MouseDown(BPoint where);
	
			void		SetMiniTitle(const char *title) const;
			void		SetMiniIcon(const BBitmap *bmp = NULL);
			void		SetSmallMiniIcon(const BBitmap *bmp = NULL);
	
protected:
			DFWindow	*fBig;
			BBitmap		*fIcon;
			BBitmap		*fSmallIcon;
			BStringView	*fTitle;
};


inline
void
DFMiniView::SetMiniTitle(const char *title) const
{
	fTitle->SetText(title);
}


// class DFMiniWindow

#pragma mark ## MINI_WINDOW

class
DFMiniWindow : public BWindow
{

public:
						DFMiniWindow(
							DFWindow *bigWindow,
							ulong workspaces);
							
	virtual				~DFMiniWindow() {}
	
	virtual bool		QuitRequested(void) { return FALSE; }
	virtual void		Show(void);
	virtual void		Hide(void);
	virtual void		WindowActivated(bool flag);
	
			void		SetMiniTitle(const char *title);
			void		SetMiniIcon(const BBitmap *bmp = NULL);
			void		SetSmallMiniIcon(const BBitmap *bmp = NULL);
	
	virtual void		WorkspacesChanged(ulong oldWS, ulong newWS);
	
protected:
			BRect		GetRect(void);

			DFWindow	*fBig;
};	


inline
void
DFMiniWindow::SetMiniTitle(const char *title)
{
	Lock();
	SetTitle(title);
	static_cast<DFMiniView*>(ChildAt(0))->SetMiniTitle(title);
	Unlock();
}


inline
void
DFMiniWindow::SetMiniIcon(const BBitmap *bmp)
{
	Lock();
	static_cast<DFMiniView*>(ChildAt(0))->SetMiniIcon(bmp);
	Unlock();
}


inline
void
DFMiniWindow::SetSmallMiniIcon(const BBitmap *bmp)
{
	Lock();
	static_cast<DFMiniView*>(ChildAt(0))->SetSmallMiniIcon(bmp);
	Unlock();
}



// class DFFilter

#pragma mark ## FILTER

class
DFFilter : public BMessageFilter
{

public:
								DFFilter(void);
		virtual					~DFFilter() {}
		
		virtual	filter_result	Filter(BMessage *, BHandler **);
};


// class DFWindow

#pragma mark ## WINDOW

class 
DFWindow : public BWindow 
{

public:
						DFWindow(
							BRect frame, 
							const char *title,
							window_type type, 
							ulong flags, 
							ulong workspaces = B_CURRENT_WORKSPACE); 
							
	virtual 			~DFWindow();
	
	virtual void		Minimize(bool minimize);
	virtual void		Hide(void);
	virtual void		Show(void);
	virtual void		WindowActivated(bool flag);
	
			void		SetMiniTitle(const char *title) const;
			void		SetMiniIcon(const BBitmap *bmp = NULL) const;
			void		SetSmallMiniIcon(const BBitmap *bmp = NULL) const;
	
			bool		IsMinimized(void) const { return fMinimized; }
	
	virtual void		WorkspacesChanged(ulong oldWS, ulong newWS);
	
protected:
	bool				fMinimized;
	DFMiniWindow		*fMini;
	DFFilter			*fFilter;
};


inline
void
DFWindow::SetMiniTitle(const char *title) const
{
	if (fMini) fMini->SetMiniTitle(title);
}


inline
void
DFWindow::SetMiniIcon(const BBitmap *bmp) const
{
	if (fMini) fMini->SetMiniIcon(bmp);
}


inline
void
DFWindow::SetSmallMiniIcon(const BBitmap *bmp) const
{
	if (fMini) fMini->SetSmallMiniIcon(bmp);
}

#endif
